!********************************************************************************
!** FICHE F.4.  VELOCITY VERSION OF VERLET ALGORITHM                           **
!** This FORTRAN code is intended to illustrate points made in the text.       **
!** To our knowledge it works correctly.  However it is the responsibility of  **
!** the user to test it, if it is to be used in a research application.        **
!   (Modified by Yoshi, Jan 2005)
!********************************************************************************

!C    *******************************************************************
!C    ** TWO ROUTINES THAT TOGETHER IMPLEMENT VELOCITY VERLET METHOD.  **
!C    **                                                               **
!C    ** REFERENCE:                                                    **
!C    **                                                               **
!C    ** SWOPE ET AL., J. CHEM. PHYS. 76, 637, 1982.                   **
!C    **                                                               **
!C    ** ROUTINES SUPPLIED:                                            **
!C    **                                                               **
!C    ** SUBROUTINE MOVEA ( DT, M )                                    **
!C    **    MOVES POSITIONS AND PARTIALLY UPDATES VELOCITIES.          **
!C    ** SUBROUTINE MOVEB ( DT, M, K )                                 **
!C    **    COMPLETES VELOCITY MOVE AND CALCULATES KINETIC ENERGY.     **
!C    **                                                               **
!C    ** PRINCIPAL VARIABLES:                                          **
!C    **                                                               **
!C    ** INTEGER N                   NUMBER OF MOLECULES               **
!C    ** DP      DT                  TIMESTEP                          **
!C    ** DP      WM                  ATOMIC MASS                       **
!C    ** DP      K                   KINETIC ENERGY                    **
!C    ** DP      RX(N),RY(N),RZ(N)   POSITIONS                         **
!C    ** DP      VX(N),VY(N),VZ(N)   VELOCITIES                        **
!C    ** DP      FX(N),FY(N),FZ(N)   FORCES                            **
!C    **                                                               **
!C    ** USAGE:                                                        **
!C    **                                                               **
!C    ** AT THE START OF A TIMESTEP, MOVEA IS CALLED TO ADVANCE THE    **
!C    ** POSITIONS AND 'HALF-ADVANCE' THE VELOCITIES.  THEN THE FORCE  **
!C    ** ROUTINE IS CALLED, AND THIS IS FOLLOWED BY MOVEB WHICH        **
!C    ** COMPLETES THE ADVANCEMENT OF VELOCITIES.                      **
!C    *******************************************************************



!    *******************************************************************
!    ** FIRST PART OF VELOCITY VERLET ALGORITHM                       **
!    **                                                               **
!    ** USAGE:                                                        **
!    **                                                               **
!    ** THE FIRST PART OF THE ALGORITHM IS A TAYLOR SERIES WHICH      **
!    ** ADVANCES POSITIONS FROM T TO T + DT AND VELOCITIES FROM       **
!    ** T TO T + DT/2.  AFTER THIS, THE FORCE ROUTINE IS CALLED.      **
!    *******************************************************************

        SUBROUTINE velocity_verlet_a
        use variables
        use constants
        implicit double precision (a-h,o-z)

        DT2   = DT / 2.0
        DTSQ2 = DT * DT2

        DO 100 I = 1, N

           RX(I) = RX(I) + DT * VX(I) + DTSQ2 * FX(I) / WM(I)
           RY(I) = RY(I) + DT * VY(I) + DTSQ2 * FY(I) / WM(I)
           RZ(I) = RZ(I) + DT * VZ(I) + DTSQ2 * FZ(I) / WM(I)
           VX(I) = VX(I) + DT2 * FX(I) / WM(I)
           VY(I) = VY(I) + DT2 * FY(I) / WM(I)
           VZ(I) = VZ(I) + DT2 * FZ(I) / WM(I)

100     CONTINUE

        RETURN
        END

!C    *******************************************************************
!C    ** SECOND PART OF VELOCITY VERLET ALGORITHM                      **
!C    **                                                               **
!C    ** USAGE:                                                        **
!C    **                                                               **
!C    ** THE SECOND PART OF THE ALGORITHM ADVANCES VELOCITIES FROM     **
!C    ** T + DT/2 TO T + DT. THIS ASSUMES THAT FORCES HAVE BEEN        **
!C    ** COMPUTED IN THE FORCE ROUTINE AND STORED IN FX, FY, FZ.       **
!C    *******************************************************************

        SUBROUTINE velocity_verlet_b
        use variables
        use constants
        implicit double precision (a-h,o-z)

        DT2 = DT / 2.0

        DO 200 I = 1, N

           VX(I) = VX(I) + DT2 * FX(I) / WM(I)
           VY(I) = VY(I) + DT2 * FY(I) / WM(I)
           VZ(I) = VZ(I) + DT2 * FZ(I) / WM(I)

200     CONTINUE

        RETURN
        END
